// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// Copyright (C) 2012-2014	JoostinOnline

#include <gccore.h>
#include <grrlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <wiiuse/wpad.h>
#include <fat.h>

#include "font_ttf.h"
#include "background_jpg.h"

#define VERSION			"1.8"
#define FONTSIZE_TITLE	28
#define FONTSIZE_SUB	20
#define FONTSIZE_REMOTE	25
#define INDENT_REMOTE	90
#define INDENT_TITLE	15
//#define SCREENSHOT
//#define DEMO_MODE

// RGBA Colors
#define BLACK		0x000000FF
#define MAROON		0x800000FF
#define GREEN		0x00C108FF
#define OLIVE		0x808000FF
#define NAVY		0x000080FF
#define PURPLE		0x800080FF
#define TEAL		0x008080FF
#define GRAY		0x808080FF
#define SILVER		0xC0C0C0FF
#define RED			0xFF0000FF
#define LIME		0x00FF00FF
#define YELLOW		0xFFFF00FF
#define BLUE		0x0000FFFF
#define FUCHSIA		0xFF00FFFF
#define AQUA		0x00FFFFFF
#define WHITE		0xFFFFFFFF

extern void __exception_setreload(int t);

s8 HWButton = -1;
static const int row[] = {10, 47, 40, 80, 110, 173, 236, 299, 362, 440};
GRRLIB_ttfFont *myFont;
GRRLIB_texImg *bkg;
GRRLIB_texImg *ScreenBuf;

enum {
	TITLE,
	SUBTEXT,
	ERROR,
	EXIT,
	REMOTE1,
	REMOTE2,
	REMOTE3,
	REMOTE4,
	REMOTE5,
	PRESS_ANY_BUTTON
};

//---------------------------------------------------------------------------------
void WiiPowerPressed(void) {
//---------------------------------------------------------------------------------
	HWButton = SYS_POWEROFF;
}

//---------------------------------------------------------------------------------
void WiiResetPressed(void) {
//---------------------------------------------------------------------------------
	HWButton = SYS_RESTART;
}

//---------------------------------------------------------------------------------
u32 ButtonsDown(void) {
//---------------------------------------------------------------------------------
	WPAD_ScanPads();
	return WPAD_ButtonsDown(0) + WPAD_ButtonsDown(1) + WPAD_ButtonsDown(2) + WPAD_ButtonsDown(3) + WPAD_ButtonsDown(4);
}

//---------------------------------------------------------------------------------
void BatteryLevel(u8 battery, int x, int y) {
//---------------------------------------------------------------------------------
	char buffer[20];
	if (battery) {
		if (battery > 100) battery = 100;

		sprintf(buffer, " %i%%", battery);
		GRRLIB_PrintfTTF(x, y, myFont, buffer, FONTSIZE_REMOTE, GREEN);
	} else {
		GRRLIB_PrintfTTF(x, y, myFont, " Not connected", FONTSIZE_REMOTE, RED);
	}
}

void LoadBackground(void) {
	fatInitDefault();
	bkg = GRRLIB_LoadTextureFromFile("background.png");
	if (bkg == NULL) bkg = GRRLIB_LoadTextureJPG(background_jpg); // Default background image
	return;
}

//---------------------------------------------------------------------------------
void Deinit(void) {
//---------------------------------------------------------------------------------
	GRRLIB_FreeTTF(myFont);
	GRRLIB_FreeTexture(bkg);
	GRRLIB_FreeTexture(ScreenBuf);
	GRRLIB_Exit();
}

//---------------------------------------------------------------------------------
void Init() {
//---------------------------------------------------------------------------------
	GRRLIB_Init();
	myFont = GRRLIB_LoadTTF(font_ttf, font_ttf_size); // Truetype font
	if (WPAD_Init() == WPAD_ERR_NONEREGISTERED) {
		GRRLIB_PrintfTTF(INDENT_TITLE, row[ERROR], myFont, "Error: No remotes have been registered", FONTSIZE_TITLE, RED);
		GRRLIB_PrintfTTF(INDENT_TITLE, row[EXIT], myFont, "Exiting...", FONTSIZE_TITLE, RED);
		GRRLIB_Render();
		sleep(4);
		Deinit();
		exit(0);
	}
	#ifndef DEMO
	WPAD_SetIdleTimeout(60 * 2); // Remote will turn off after 2 minutes of inactivity.
	#endif

	SYS_SetPowerCallback(WiiPowerPressed);
	SYS_SetResetCallback(WiiResetPressed);
	
	LoadBackground();
	
	ScreenBuf = GRRLIB_CreateEmptyTexture(rmode->fbWidth, rmode->efbHeight);
	
	
}

//---------------------------------------------------------------------------------
int main(int argc, char **argv) {
//---------------------------------------------------------------------------------
	__exception_setreload(0); // In the event of a code dump, app will exit immediately
	Init();

	while(true) {
		if (ButtonsDown()) {
			Deinit();
			exit(0);
		}
		// Write constants to screen
		GRRLIB_DrawImg(0, 0, bkg, 0, 1, 1, RGBA(255, 255, 255, 255));
		GRRLIB_PrintfTTF(INDENT_TITLE, row[TITLE], myFont, "PowerCheck v" VERSION " by JoostinOnline", FONTSIZE_TITLE, WHITE);
		GRRLIB_PrintfTTF(INDENT_TITLE, row[SUBTEXT], myFont, "www.HacksDen.com", FONTSIZE_SUB, WHITE);
	
		
		
		#ifdef DEMO_MODE
		BatteryLevel(100, INDENT_REMOTE, row[REMOTE1]);
		BatteryLevel(52, INDENT_REMOTE, row[REMOTE2]);
		BatteryLevel(73, INDENT_REMOTE, row[REMOTE3]);
		BatteryLevel(0, INDENT_REMOTE, row[REMOTE4]);
		BatteryLevel(88, INDENT_REMOTE, row[REMOTE5]);
		#else
		BatteryLevel(WPAD_BatteryLevel(0) / 1.5, INDENT_REMOTE, row[REMOTE1]);
		BatteryLevel(WPAD_BatteryLevel(1) / 1.5, INDENT_REMOTE, row[REMOTE2]);
		BatteryLevel(WPAD_BatteryLevel(2) / 1.5, INDENT_REMOTE, row[REMOTE3]);
		BatteryLevel(WPAD_BatteryLevel(3) / 1.5, INDENT_REMOTE, row[REMOTE4]);
		BatteryLevel(WPAD_BatteryLevel(4) / 3.0, INDENT_REMOTE, row[REMOTE5]); // Balance Board reports twice the percentage
		#endif
		
		GRRLIB_PrintfTTF(INDENT_TITLE, row[PRESS_ANY_BUTTON], myFont, "Press any button to exit", FONTSIZE_TITLE, WHITE);
		
		if(HWButton != -1)
		{
			#ifdef SCREENSHOT
			GRRLIB_ScrShot("Screenshot.png");  //Takes a screenshot if you press POWER or RESET
			#endif
			Deinit();
			SYS_ResetSystem(HWButton, 0, 0);
		}
		// Work-around for GRRLIB bug
		GRRLIB_Screen2Texture(0, 0, ScreenBuf, GX_FALSE);
	
		GRRLIB_DrawImg(0, 0, ScreenBuf, 0, 1, 1, RGBA(255, 255, 255, 255));

		VIDEO_WaitVSync();
		GRRLIB_Render();
	}
}
